"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const moment_1 = __importDefault(require("moment"));
const testrail_js_api_1 = require("testrail-js-api");
const loadJSON = (file) => {
    const data = fs_1.default.readFileSync(file, { encoding: "utf8" });
    if (data) {
        return JSON.parse(data);
    }
    return {};
};
const prepareConfig = (options) => {
    const config = Object.assign(loadJSON(path_1.default.join(process.cwd(), ".testrailrc")), options);
    return {
        enabled: process.env.TESTRAIL_ENABLED === "true" || config.enabled || false,
        host: process.env.TESTRAIL_HOST || config.host,
        user: process.env.TESTRAIL_USER || config.user,
        apiKey: process.env.TESTRAIL_API_KEY || config.apiKey,
        projectId: Number((process.env.TESTRAIL_PROJECT_ID || config.projectId || "").replace("P", "").trim()),
        suiteId: Number((process.env.TESTRAIL_SUITE_ID || config.suiteId || "").replace("S", "").trim()),
        coverageCaseId: Number((process.env.TESTRAIL_COVERAGE_CASE_ID || config.coverageCaseId || "").replace("C", "").trim()),
        runName: process.env.TESTRAIL_RUN_NAME || config.runName || "%BRANCH%#%BUILD% - %DATE%",
        runDescription: process.env.TESTRAIL_RUN_DESCRIPTION || config.runDescription,
        reference: process.env.TESTRAIL_REFERENCE || config.reference,
        branchEnv: process.env.TESTRAIL_BRANCH_ENV || config.branchEnv || "BRANCH",
        buildNoEnv: process.env.TESTRAIL_BUILD_NO_ENV || config.buildNoEnv || "BUILD_NUMBER",
        dateFormat: process.env.TESTRAIL_DATE_FORMAT || config.dateFormat || "YYYY-MM-DD HH:mm:ss",
        runCloseAfterDays: Number(process.env.TESTRAIL_RUN_CLOSE_AFTER_DAYS || config.runCloseAfterDays) || 0,
    };
};
const prepareReportName = (config, branch, buildNo) => {
    const date = moment_1.default().format(config.dateFormat);
    return config.runName.replace("%BRANCH%", branch).replace("%BUILD%", buildNo).replace("%DATE%", date);
};
const prepareReference = (config, branch, buildNo) => {
    return config.reference ? config.reference.replace("%BRANCH%", branch).replace("%BUILD%", buildNo) : "";
};
const prepareReport = (results) => {
    var _a;
    const { numTotalTests, numPassedTests, numPendingTests, numFailedTests, numTotalTestSuites, numPassedTestSuites, numPendingTestSuites, numFailedTestSuites, } = results;
    const { branches, functions, lines, statements } = ((_a = results === null || results === void 0 ? void 0 : results.coverageMap) === null || _a === void 0 ? void 0 : _a.getCoverageSummary().data) || {};
    let report = `# Unit / Component test results:
||| Type       | Total | Passed | Skipped | Failed
|| Test Suites | ${numTotalTestSuites} | ${numPassedTestSuites} | ${numPendingTestSuites} | ${numFailedTestSuites}
|| Tests       | ${numTotalTests} | ${numPassedTests} | ${numPendingTests} | ${numFailedTests}`;
    if (branches && branches.pct && functions && statements && lines) {
        report += `

# Unit tests Coverage:

||| Type      | Percentage | Total | Covered | Skipped
|| functions  | ${functions.pct}% | ${functions.total} | ${functions.covered} | ${functions.skipped}
|| statements | ${statements.pct}% | ${statements.total} | ${statements.covered} | ${statements.skipped}
|| lines      | ${lines.pct}% | ${lines.total} | ${lines.covered} | ${lines.skipped}
|| branches   | ${branches.pct}% | ${branches.total} | ${branches.covered} | ${branches.skipped}`;
    }
    return report;
};
const verifyConfig = (config) => {
    const { enabled, host, user, apiKey, projectId, suiteId, coverageCaseId } = config;
    if (enabled) {
        if (!host) {
            console.log("[TestRail] Hostname was not provided.");
        }
        if (!user || !apiKey) {
            console.log("[TestRail] Username or api key was not provided.");
        }
        if (!projectId) {
            console.log("[TestRail] Project id was not provided.");
        }
        if (!coverageCaseId) {
            console.log("[TestRail] Coverage testcase id was not provided.");
        }
        if (!suiteId) {
            console.log("[TestRail] Suite id was not provided.");
        }
        if (host && user && apiKey && projectId && coverageCaseId && suiteId) {
            return true;
        }
    }
    return false;
};
const throwOnApiError = async (apiResult) => {
    const { response, value } = (await apiResult);
    if (response.status >= 400) {
        console.error("[TestRail] Error during API request");
        throw {
            url: response.url,
            status: response.status,
            message: value,
        };
    }
    return Promise.resolve({ response, value });
};
class JestTestrailReporter {
    constructor(globalConfig, options) {
        this.config = prepareConfig(options);
        this.branch = process.env[this.config.branchEnv] || "master";
        this.buildNo = process.env[this.config.buildNoEnv] || "unknown";
    }
    async onRunComplete(contexts, results) {
        const { host, user, apiKey, projectId, suiteId, coverageCaseId, runDescription } = this.config;
        if (verifyConfig(this.config)) {
            try {
                const testrail = new testrail_js_api_1.TestRail(host, user, apiKey);
                const name = prepareReportName(this.config, this.branch, this.buildNo);
                const refs = prepareReference(this.config, this.branch, this.buildNo);
                const report = prepareReport(results);
                const { value: runs } = await throwOnApiError(testrail.getRuns(projectId, { is_completed: 0 }));
                const existingRun = runs === null || runs === void 0 ? void 0 : runs.find((run) => run.refs === refs);
                let run;
                if (existingRun) {
                    run = existingRun;
                    const { value: tests } = await throwOnApiError(testrail.getTests(existingRun.id));
                    const currentCaseIds = (tests === null || tests === void 0 ? void 0 : tests.map((test) => test.case_id)) || [];
                    const additionalDescription = "\n" + runDescription || report;
                    await throwOnApiError(testrail.updateRun(existingRun.id, {
                        description: existingRun.description.replace(additionalDescription, "") + additionalDescription,
                        case_ids: [...currentCaseIds, coverageCaseId],
                    }));
                    console.log(`[TestRail] Test run updated successfully: ${name}`);
                }
                else {
                    const payload = {
                        suite_id: suiteId,
                        include_all: false,
                        case_ids: [coverageCaseId],
                        name,
                        description: runDescription || report,
                        refs,
                    };
                    const { value: newRun } = await throwOnApiError(testrail.addRun(this.config.projectId, payload));
                    run = newRun;
                    console.log(`[TestRail] Test run added successfully: ${name}`);
                }
                if (run === null || run === void 0 ? void 0 : run.id) {
                    const { value: resultArray } = await throwOnApiError(testrail.addResultsForCases(run.id, [
                        {
                            case_id: coverageCaseId,
                            status_id: results.numFailedTests ? 5 : 1,
                            comment: report,
                        },
                    ]));
                    if (resultArray.length) {
                        console.log("[TestRail] Sending report to TestRail successfull");
                    }
                }
            }
            catch (error) {
                console.error("[TestRail] Sending report to TestRail failed", error);
            }
        }
    }
}
exports.default = JestTestrailReporter;
